import { HttpClient, HttpErrorResponse } from '@angular/common/http';
import { Component, Input, OnInit, signal } from '@angular/core';
import { Router } from '@angular/router';
import { BehaviorSubject, lastValueFrom } from 'rxjs';
import { Employer } from '../../models/employer.model';
import { Job } from '../../models/job.model';
import { User } from '../../models/user.model';
import { AuthService } from '../../services/auth.service';
import { NotificationService } from '../../services/notification.service';
import { CustomRegExp } from '../../utils/CustomRegExp';
import { CanExitResolver, Utils } from '../../utils/utils';
import { ProfileService } from '../services/profile.service';

enum tabsEnum {
  details,
  credentials,
  employer,
}

enum showPasswordEnum {
  oldPassword = 'oldPassword',
  newPassword = 'newPassword',
  newPasswordConfirm = 'newPasswordConfirm',
}

type passwordStatuses = 'error' | 'info' | 'success';

@Component({
  selector: 'app-edit',
  templateUrl: './edit.component.html',
  styleUrls: ['./edit.component.scss'],
})
export class EditComponent implements OnInit {
  public tabsEnum = tabsEnum;
  public currentTab = tabsEnum.details;

  @Input() userProfile: User;
  public initialUserProfile: User;
  public newEmail = '';
  public newEmailConfirm = '';
  public oldPassword = '';
  public newPassword = '';
  public newPasswordConfirm = '';
  public isShowPassword = {
    oldPassword: false,
    newPassword: false,
    newPasswordConfirm: false,
  };
  public ShowPassword = showPasswordEnum;
  public selectedJob: Job;
  public jobName: string;
  public isNewJob: boolean;
  public selectedEmployer: Employer;
  public employerName: string;
  public isNewEmployer: boolean;
  public isAlreadySearching = false;
  public hasPersonalOffer = false;
  public errorChangeEmail = false;
  public statusCode = 200;
  public avatarsCount = 0;

  private selectedRdvChoiceSubject = new BehaviorSubject<boolean>(false);
  selectedRdvChoice$ = this.selectedRdvChoiceSubject.asObservable();
  private currentToastId: number | null = null;

  // Modal canExit var
  public isEmailModalOpen = signal(false);
  public isPasswordModalOpen = signal(false);
  public isDeleteAccountModalOpen = signal(false);
  public isPendingChangesModalOpen = signal(false);
  private resolve: CanExitResolver;
  private canDeactivate = false;
  public isAppointmentModalOpen = signal(false);
  public isAvatarModalOpen = signal(false);
  public selectedAvatarIndex: number | null = null;
  public currentAvatar: string | null;

  constructor(
    public profileService: ProfileService,
    private notificationService: NotificationService,
    private authService: AuthService,
    private utils: Utils,
    private router: Router,
    private http: HttpClient,
  ) {
    this.getAvatarsCount();
  }

  ngOnInit(): void {
    this.profileService.getProfile().then((profile) => {
      if (profile.hasOwnProperty('withAppointment')) this.selectedRdvChoiceSubject.next(profile.withAppointment);
      this.userProfile = new User(profile);
      this.userProfile.description = this.userProfile.description || '';
      this.initialUserProfile = new User({ ...profile });
      this.selectedEmployer = { ...profile.employer };
      this.employerName = this.selectedEmployer?.name;
      this.selectedJob = { ...profile.job };
      this.jobName = this.selectedJob?.name;
      this.hasPersonalOffer = profile.job?.hasPersonalOffer;
      this.currentAvatar = profile.avatar || 'avatar1';
    });
  }

  public goBack(): void {
    history.back();
  }

  public getTabsNames(): string[] {
    return ['Informations', 'Email et mot de passe', 'Employeur et fonction'];
  }

  public getTabsIds(): string[] {
    return ['detailsTab', 'credentials', 'employerJob'];
  }

  public changeTab(tab: number): void {
    this.currentTab = tab;
  }

  public phoneValid(): boolean {
    return Boolean(this.userProfile.phone.match(CustomRegExp.PHONE));
  }

  public nameValid(): boolean {
    return this.userProfile.name !== '';
  }

  public surnameValid(): boolean {
    return this.userProfile.surname !== '';
  }

  public descriptionValid(): boolean {
    return this.userProfile.description?.length <= 500;
  }

  public emailValid(email: string): boolean {
    if (email === this.userProfile.email) return false;
    return Boolean(RegExp(CustomRegExp.EMAIL).exec(email));
  }

  public getNewEmailStatus(): boolean {
    return this.emailValid(this.newEmail);
  }

  public getNewEmailStatusText(): string {
    if (this.newEmail === this.userProfile.email) {
      return "L'email doit être différent du précédent";
    }
    return `Nouvel email ${this.getNewEmailStatus() ? 'valide' : 'invalide'}`;
  }

  public getNewEmailConfirmStatus(): boolean {
    return this.emailValid(this.newEmail) && this.newEmailConfirm === this.newEmail;
  }

  public getNewEmailConfirmStatusText(): string {
    if (!this.newEmail || !this.newEmailConfirm) return '';
    if (this.newEmailConfirm === this.userProfile.email) {
      return "L'email doit être différent du précédent";
    }
    return `Confirmation du nouvel email ${this.getNewEmailConfirmStatus() ? 'valide' : 'invalide'}`;
  }

  public passwordValid(password: string): boolean {
    return !!(
      password.match(CustomRegExp.SPECHAR) &&
      password.match(CustomRegExp.DIGIT) &&
      password.match(CustomRegExp.UPPERCASE) &&
      password.match(CustomRegExp.LOWERCASE) &&
      password.match(CustomRegExp.MINLENGTH)
    );
  }

  public passwordValidConfirm(passwordConfirm: string, password: string): boolean {
    return this.passwordValid(passwordConfirm) && passwordConfirm === password;
  }

  public getPasswordStatus(password: string, compareTo?: string): passwordStatuses {
    if (this.utils.isNullOrEmpty(password)) return 'info';

    const isValid = compareTo ? this.passwordValidConfirm(compareTo, password) : this.passwordValid(password);
    return isValid ? 'success' : 'error';
  }

  public getOldPasswordStatusText(status: passwordStatuses): string {
    const statusTexts = {
      error: 'Mot de passe invalide.',
      success: 'Champ rempli.',
    };

    return statusTexts[status];
  }

  public getPasswordStatusText(status: passwordStatuses): string {
    const statusTexts = {
      info: 'Le mot de passe doit obligatoirement contenir : 8 caractères, une majuscule, un caractère spécial.',
      error: 'Le mot de passe doit obligatoirement contenir : 8 caractères, une majuscule, un caractère spécial.',
      success: 'Mot de passe valide.',
    };

    return statusTexts[status];
  }

  public getNewPasswordConfirmStatusText(status: passwordStatuses): string {
    const statusTexts = {
      error: 'Confirmation du nouveau mot de passe invalide.',
      success: 'Confirmation du nouveau mot de passe valide.',
    };

    return statusTexts[status];
  }

  getStatus(success: boolean): 'error' | 'success' {
    return success ? 'success' : 'error';
  }

  public cancel(): void {
    this.userProfile = { ...this.initialUserProfile };
  }

  public showPassword(key: showPasswordEnum): void {
    this.isShowPassword[key] = !this.isShowPassword[key];
  }

  public closeModal(): void {
    this.oldPassword = '';
    this.newPassword = '';
    this.newPasswordConfirm = '';
    this.isEmailModalOpen.set(false);
    this.isPasswordModalOpen.set(false);
    this.isDeleteAccountModalOpen.set(false);
  }

  public isPageValid(): boolean {
    if (this.currentTab === tabsEnum.details) {
      return (
        this.coordsHaveChanged() &&
        this.phoneValid() &&
        this.nameValid() &&
        this.surnameValid() &&
        this.descriptionValid()
      );
    } else if (this.currentTab === tabsEnum.credentials) {
      if (this.isEmailModalOpen()) {
        return this.emailValid(this.newEmail) && this.newEmail === this.newEmailConfirm;
      } else if (this.isPasswordModalOpen()) {
        return this.passwordValid(this.newPassword) && this.newPassword === this.newPasswordConfirm;
      }
    } else if (this.currentTab === tabsEnum.employer) {
      return this.employerAndJobValid() && this.employerOrJobHasChanged();
    }
  }

  public confirm(): void {
    this.canDeactivate = true;
    if (this.currentTab === tabsEnum.details) {
      this.confirmDetails();
      this.confirmDescription();
    } else if (this.currentTab === tabsEnum.credentials) {
      if (this.isEmailModalOpen()) {
        this.confirmNewEmail();
      } else if (this.isPasswordModalOpen()) {
        this.confirmNewPassword();
      }
    } else if (this.currentTab === tabsEnum.employer) {
      this.confirmEmployer();
    }
  }

  public confirmDetails(): void {
    this.profileService
      .updateDetails({
        name: this.userProfile.name,
        surname: this.userProfile.surname,
        phone: this.userProfile.phone,
      })
      .subscribe((user: User) => {
        this.notificationService.showSuccess('Vos informations ont bien été enregistrées');
        // Update localStorage
        const updatedUser = {
          ...this.authService.userSubject.value,
          name: user.name,
          surname: user.surname,
          phone: user.phone,
        };
        localStorage.setItem('user', JSON.stringify(updatedUser));
        this.authService.userSubject.next(updatedUser);
        this.updateInitialProfile();
        this.router.navigate(['/profil']);
      });
  }

  public confirmDescription(): void {
    this.profileService.updateDescription(this.userProfile.description).subscribe(() => {
      this.updateInitialProfile();
    });
  }

  public async confirmEmployer(): Promise<void> {
    if (this.isNewJob) {
      await lastValueFrom(this.profileService.createJob(this.selectedJob));
    }
    if (this.isNewEmployer) {
      await lastValueFrom(this.profileService.createEmployer(this.selectedEmployer));
    }

    this.profileService
      .updateProfile(this.selectedEmployer.name, this.selectedJob.name, this.selectedRdvChoiceSubject.getValue())
      .subscribe({
        next: () => {
          this.router.navigate(['/profil']);
          this.notificationService.showSuccess('Vos informations ont bien été enregistrées');
        },
        error: () => {
          this.notificationService.showError("Vos informations n'ont pas été enregistrées", 'Une erreur est survenue');
          this.canDeactivate = false;
        },
      });
  }

  public onRdvRadioChange(value: boolean): void {
    if (this.userProfile.personalOffers?.length === 0 && value === true) {
      const profileUrl = this.router.serializeUrl(this.router.createUrlTree(['/profil']));
      const errorMessage = `Vous n'avez pas d'offre personnelle, pour sélectionner cette option merci d'en créer une en modifiant une structure dans <a href="${profileUrl}">votre profil</a>`;
      this.selectedRdvChoiceSubject.next(false);
      // Remove existing toast if there is one
      if (this.currentToastId !== null) {
        this.notificationService.removeToast(this.currentToastId);
      }
      // Show a new error toast and store its ID
      const newToast = this.notificationService.showError(null, errorMessage, 0, { enableHtml: true });
      this.currentToastId = newToast.toastId;
    } else {
      this.selectedRdvChoiceSubject.next(value);
    }
  }

  // Updates initialProfile so the button "Valider" is disabled after the user sends new values
  private updateInitialProfile(): void {
    this.initialUserProfile = { ...this.userProfile };
  }

  public confirmNewEmail(): void {
    if (this.emailValid(this.newEmail) && this.newEmail === this.newEmailConfirm) {
      this.profileService.changeEmail(this.newEmail, this.userProfile.email).subscribe({
        next: () => {
          this.closeModal();
          this.notificationService.showSuccess('Veuillez confirmer votre nouvelle adresse grâce au mail envoyé');
          this.newEmail = '';
          this.newEmailConfirm = '';
        },
        error: (error: HttpErrorResponse) => {
          this.statusCode = error.status;
          this.closeModal();
          this.errorChangeEmail = true;
        },
        complete: () => {
          this.canDeactivate = false;
        },
      });
    }
  }

  public confirmNewPassword(): void {
    if (this.passwordValid(this.newPassword) && this.newPassword === this.newPasswordConfirm) {
      this.profileService.changePassword(this.newPassword, this.oldPassword).subscribe({
        next: () => {
          this.notificationService.showSuccess('Votre mot de passe a bien été modifié');
          this.closeModal();
        },
        error: (error: HttpErrorResponse) => {
          if (error.status === 401) {
            this.notificationService.showError('Ancien mot de passe invalide');
          } else {
            this.notificationService.showErrorPleaseRetry('Une erreur est survenue');
          }
        },
        complete: () => {
          this.oldPassword = '';
          this.isShowPassword.oldPassword = false;
          this.canDeactivate = false;
        },
      });
    }
  }

  public async confirmDeleteAccount(): Promise<void> {
    try {
      await lastValueFrom(this.authService.login(this.userProfile.email, this.oldPassword));

      await lastValueFrom(this.profileService.deleteProfile());
      this.closeModal();

      this.notificationService.showSuccess('Votre compte a bien été supprimé');
      setTimeout(() => this.authService.logout(), 3000); // Let time for user to view notification before logout
    } catch (e) {
      if (e.status === 401) {
        this.notificationService.showError('Mot de passe invalide');
      } else {
        this.notificationService.showErrorPleaseRetry('Une erreur est survenue');
      }
    }
  }

  // Jobs
  public selectJob(job: Job): void {
    this.selectedJob = job;
    this.jobName = this.selectedJob?.name;
    this.isNewJob = false;
    this.hasPersonalOffer = job?.hasPersonalOffer;
  }

  public onCreateValueChangeJob(value: string): void {
    this.selectedJob = new Job({ name: value, validated: false, hasPersonalOffer: true });
    this.isNewJob = true;
    this.hasPersonalOffer = this.selectedJob?.hasPersonalOffer;
  }

  // Employer
  public selectEmployer(employer: Employer): void {
    this.selectedEmployer = employer;
    this.employerName = this.selectedEmployer?.name;
    this.isNewEmployer = false;
  }

  public onCreateValueChangeEmployer(value: string): void {
    this.selectedEmployer = new Employer({ name: value, validated: false });
    this.isNewEmployer = true;
  }

  public canExit(): Promise<boolean> {
    if (this.hasPendingChanges() && !this.canDeactivate) {
      return new Promise((resolve) => this.showPendingChangesModal(resolve));
    }
    if (this.hasPersonalOffer && this.selectedRdvChoiceSubject.getValue() === undefined) {
      return new Promise(() => this.isAppointmentModalOpen.set(true));
    }
    return Promise.resolve(true);
  }

  private coordsHaveChanged(): boolean {
    return (
      this.userProfile.name !== this.initialUserProfile.name ||
      this.userProfile.surname !== this.initialUserProfile.surname ||
      this.userProfile.phone !== this.initialUserProfile.phone ||
      this.descriptionHasChanged()
    );
  }

  private employerAndJobValid(): boolean {
    // Check if mandatory fields are set (with rdv defined if needed)
    if (!this.selectedEmployer || !this.selectedJob) {
      return false;
    }
    if (this.hasPersonalOffer && this.selectedRdvChoiceSubject.getValue() === undefined) {
      return false;
    }
    return true;
  }

  private employerOrJobHasChanged(): boolean {
    // Check if any field has changed : employer or job, or rdv choice if applicable
    return (
      this.selectedEmployer?.name !== this.initialUserProfile.employer?.name ||
      this.selectedJob?.name !== this.initialUserProfile.job?.name ||
      (this.hasPersonalOffer && this.selectedRdvChoiceSubject.getValue() !== this.initialUserProfile.withAppointment)
    );
  }

  private descriptionHasChanged(): boolean {
    return this.userProfile.description !== (this.initialUserProfile.description ?? '');
  }

  public hasPendingChanges(): boolean {
    return this.coordsHaveChanged() || this.employerOrJobHasChanged() || this.descriptionHasChanged();
  }

  public showPendingChangesModal(resolve: CanExitResolver): void {
    this.isPendingChangesModalOpen.set(true);
    this.resolve = resolve;
  }
  public hasPendingChangesRedirectionAccepted(hasAccept: boolean): void {
    this.resolve(hasAccept);
    this.isPendingChangesModalOpen.set(false);
  }

  public openAvatarModal(): void {
    this.isAvatarModalOpen.set(true);
    // Pre select current avatar on opening modal
    this.selectedAvatarIndex = Number(this.currentAvatar?.at(-1)) - 1 || 0;
  }

  public selectAvatar(index: number): void {
    this.selectedAvatarIndex = index;
  }

  public hasChangedAvatar(hasValidated: boolean): void {
    // close modal
    this.isAvatarModalOpen.set(false);
    // save if modal validated
    if (hasValidated) {
      this.currentAvatar = 'avatar' + (Number(this.selectedAvatarIndex) + 1);
      this.userProfile.avatar = this.currentAvatar;
      this.profileService.updateAvatar(this.currentAvatar).subscribe(() => {
        this.notificationService.showSuccess('Votre nouvel avatar a bien été enregistré');
        // Update localStorage (to update header avatar)
        const updatedUser = {
          ...this.authService.userSubject.value,
          avatar: this.currentAvatar,
        };
        localStorage.setItem('user', JSON.stringify(updatedUser));
        this.authService.userSubject.next(updatedUser);
        this.updateInitialProfile();
      });
    }
  }

  // Count avatars in sprite
  private getAvatarsCount(): void {
    this.http.get('assets/avatar/sprite.svg', { responseType: 'text' }).subscribe(
      (svgContent: string) => {
        const parser = new DOMParser();
        const svgDoc = parser.parseFromString(svgContent, 'image/svg+xml');
        const symbolCount = svgDoc.querySelectorAll('symbol').length;
        this.avatarsCount = symbolCount;
      },
      (error) => {
        console.error('Error loading SVG sprite:', error);
      },
    );
  }
}
